require 'spec_helper'
include RedirHelper

describe :redirect do

  after :each do
    Redirect.delete_all
    RedirectLog.delete_all
  end

  context 'redirect generator' do
    context 'should generate a unique url' do
      before :each do
        SecureRandom.should_receive(:urlsafe_base64).with(6).and_return('eeeeee')
      end
      
      let(:redirect) { Redirect.create_shortened_url 'http://www.google.com' }
      subject {redirect}
      its (:shortened_id) {should eq 'eeeeee'}
      its (:shortened_path) {should eq 'a/eeeeee'}
      its (:shortened_url) {should eq 'http://localhost:3000/a/eeeeee'}
    end


    context 'duplicate redirects' do
      before do
        SecureRandom.should_receive(:urlsafe_base64).with(6).twice.and_return('eeeeee')
        SecureRandom.should_receive(:urlsafe_base64).with(6).once.and_return('ffffff')
      end

      let(:redirect) { Redirect.create_shortened_url 'http://www.google.com' }
      let(:second_redirect) { Redirect.create_shortened_url 'http://www.yahoo.com' }

      it "should get a unique url on each request" do
        redirect.shortened_id.should eq 'eeeeee'
        second_redirect.shortened_id.should eq 'ffffff'
      end
    end
  end

  context 'leading and trailing slashes' do
    context 'host' do
      context 'no trailing slash' do
        let (:redirect) {Redirect.create_shortened_url 'www.springbot.com'}

        before do
          Redirect.host = 'www.honeybadger.com'
          SecureRandom.should_receive(:urlsafe_base64).with(6).at_most(:once).and_return('eeeeee')
        end

        context 'global host' do
          subject {Redirect}
          its (:host) {should eq 'www.honeybadger.com/'}
        end

        subject {redirect}
        its (:shortened_url) {should eq 'www.honeybadger.com/a/eeeeee'}
      end
    end

    context 'path' do

      shared_examples_for 'a correct redirect' do
        before {SecureRandom.should_receive(:urlsafe_base64).with(6).at_most(:once).and_return('eeeeee')}
        let (:redirect) {Redirect.create_shortened_url 'www.springbot.com'}
        context 'global host' do
          subject {Redirect}
          its (:path) {should eq 'badger/'}
        end

        subject {redirect}
        its (:shortened_url) {should eq 'www.honeybadger.com/badger/eeeeee'}
        its (:shortened_path) {should eq 'badger/eeeeee'}
      end    

      context 'no trailing slash' do        
        before { Redirect.path = 'badger'}
        it_behaves_like 'a correct redirect'
      end

      context 'no trailing slash and slash at the beginning' do        
        before { Redirect.path = '/badger' }
        it_behaves_like 'a correct redirect'
      end

    end
  end
end
